package com.qmth.sop.business.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.sop.business.activiti.service.ActivitiService;
import com.qmth.sop.business.bean.params.FlowApproveParam;
import com.qmth.sop.business.bean.params.SopApplyParam;
import com.qmth.sop.business.bean.params.SopInfoListParam;
import com.qmth.sop.business.bean.result.FlowFormWidgetResult;
import com.qmth.sop.business.bean.result.FlowTaskResult;
import com.qmth.sop.business.entity.*;
import com.qmth.sop.business.mapper.TBSopInfoMapper;
import com.qmth.sop.business.service.TBSopInfoDetailService;
import com.qmth.sop.business.service.TBSopInfoService;
import com.qmth.sop.business.service.TFCustomFlowEntityService;
import com.qmth.sop.common.contant.SystemConstant;
import com.qmth.sop.common.enums.ExceptionResultEnum;
import com.qmth.sop.common.enums.FlowApprovePassEnum;
import com.qmth.sop.common.enums.WidgetCodeEnum;
import com.qmth.sop.common.util.ServletUtil;
import org.activiti.engine.TaskService;
import org.activiti.engine.task.Task;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;

/**
 * <p>
 * sop表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2023-08-01
 */
@Service
public class TBSopInfoServiceImpl extends ServiceImpl<TBSopInfoMapper, TBSopInfo> implements TBSopInfoService {

    @Resource
    TBSopInfoMapper tbSopInfoMapper;

    @Resource
    ActivitiService activitiService;

    @Resource
    TFCustomFlowEntityService tfCustomFlowEntityService;

    @Resource
    TaskService taskService;

    @Resource
    TBSopInfoDetailService tbSopInfoDetailService;

    @Resource
    TBSopInfoService tbSopInfoService;

    /**
     * 查询动态sop表名是否存在
     *
     * @param db
     * @param tableName
     * @return
     */
    @Override
    public int selectSopDynamicExist(String db, String tableName) {
        return this.baseMapper.selectSopDynamicExist(db, tableName);
    }

    /**
     * 保存动态sop信息
     *
     * @param tfCustomFlow
     * @param code
     * @param flowId
     * @param flowTaskResult
     * @return
     */
    @Override
    @Transactional
    public Boolean saveDynamicSop(TFCustomFlow tfCustomFlow, String code, Long flowId, FlowTaskResult flowTaskResult) {
        String tableName = tfCustomFlow.getType().getTableName() + "_" + tfCustomFlow.getVersion();
        List<FlowFormWidgetResult> flowFormWidgetResultList = flowTaskResult.getFormProperty();
        Map<String, String> formFieldMap = new HashMap<>(flowFormWidgetResultList.size());
        for (FlowFormWidgetResult f : flowFormWidgetResultList) {
            if (f.getCode() != WidgetCodeEnum.FORM_GROUP_TITLE && f.getCode() != WidgetCodeEnum.ONLE_TITLE
                    && f.getCode() != WidgetCodeEnum.BUTTON && f.getCode() != WidgetCodeEnum.IMG
                    && f.getCode() != WidgetCodeEnum.LABEL && f.getCode() != WidgetCodeEnum.SIGN
                    && f.getCode() != WidgetCodeEnum.OTHER) {
                if (Objects.nonNull(f.getValue()) && f.getValue().startsWith("[") && f.getValue().endsWith("]")) {
                    f.setValue(f.getValue().replaceAll("\\[", "").replaceAll("\\]", ""));
                    formFieldMap.put(f.getFormId(), StringUtils.join(Arrays.asList(f.getValue()), SystemConstant.LIST_JOIN_SPLIT));
                } else {
                    formFieldMap.put(f.getFormId(), f.getValue());
                }
                formFieldMap.put(f.getFormId(), f.getValue());
            }
        }
        //首先查询动态表里是否有当前表单的数据，没有插入，有则更新
        int count = this.selectDynamicSop(tableName, tfCustomFlow.getFlowDeploymentId(), flowId);
        if (count == 0) {//插入操作
            //查询所有列
            List<Map<String, String>> dynamicFieldList = this.baseMapper.selectDynamicSopAll(tableName);
            Map<String, Object> map = new LinkedHashMap<>();
            if (!CollectionUtils.isEmpty(dynamicFieldList)) {
                StringJoiner stringJoinerFieldName = new StringJoiner(",");
                StringJoiner stringJoinerFieldValue = new StringJoiner(",");
                for (Map<String, String> m : dynamicFieldList) {
                    for (Map.Entry<String, String> entry : m.entrySet()) {
                        switch (entry.getValue()) {
                            case SystemConstant.TYPE:
                                map.put(entry.getValue(), tfCustomFlow.getType());
                                break;
                            case SystemConstant.FLOW_DEPLOYMENT_ID_FIELD:
                                map.put(entry.getValue(), tfCustomFlow.getFlowDeploymentId());
                                break;
                            case SystemConstant.VERSION:
                                map.put(entry.getValue(), tfCustomFlow.getVersion());
                                break;
                            case SystemConstant.CODE:
                                map.put(entry.getValue(), code);
                                break;
                            case SystemConstant.FLOW_ID:
                                map.put(entry.getValue(), flowId);
                                break;
                            default:
                                if (formFieldMap.containsKey(entry.getValue())) {
                                    map.put(entry.getValue(), formFieldMap.get(entry.getValue()));
                                } else {
                                    map.put(entry.getValue(), null);
                                }
                                break;
                        }
                    }
                }
                for (Map.Entry<String, Object> entry : map.entrySet()) {
                    stringJoinerFieldName.add(entry.getKey());
                    stringJoinerFieldValue.add(Objects.nonNull(entry.getValue()) ? "'" + entry.getValue().toString() + "'" : null);
                }
                tbSopInfoMapper.saveDynamicSop(tableName, stringJoinerFieldName.toString(), stringJoinerFieldValue.toString());
            }
        } else {//否则更新
            StringJoiner stringJoinerFieldValue = new StringJoiner(",");
            for (Map.Entry<String, String> entry : formFieldMap.entrySet()) {
                stringJoinerFieldValue.add(entry.getKey() + "='" + entry.getValue() + "'");
            }
            tbSopInfoMapper.updateDynamicSop(tableName, tfCustomFlow.getFlowDeploymentId(), flowId, stringJoinerFieldValue.toString());
        }
        return true;
    }

    /**
     * 查询动态sop数据
     *
     * @param tableName
     * @param flowDeploymentId
     * @param flowId
     * @return
     */
    @Override
    public int selectDynamicSop(String tableName, String flowDeploymentId, Long flowId) {
        return this.baseMapper.selectDynamicSop(tableName, flowDeploymentId, flowId);
    }

    /**
     * sop申请信息
     *
     * @param sopApplyParam
     * @return
     */
    @Override
    @Transactional
    public Boolean sopApply(SopApplyParam sopApplyParam) throws InterruptedException {
        if (Objects.isNull(sopApplyParam.getCrmNo())) {
            throw ExceptionResultEnum.PARAMS_ERROR.exception("crm编号不能为空");
        }
        if (sopApplyParam.getApprove() != FlowApprovePassEnum.START && sopApplyParam.getApprove() != FlowApprovePassEnum.DRAFT) {
            throw ExceptionResultEnum.PARAMS_ERROR.exception("sop申请只能为提交或草稿");
        }
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        FlowTaskResult flowTaskResult = JSONObject.parseObject(sopApplyParam.getFormProperties(), FlowTaskResult.class);
        Long regionUserId = null, engineerUserId = null;
        String assistantEngineerUserIds = null;
        this.getApproveUserIds(flowTaskResult, regionUserId, engineerUserId, assistantEngineerUserIds);

        List<String> approveUserIds = Objects.nonNull(engineerUserId) ? Arrays.asList(String.valueOf(engineerUserId)) : Arrays.asList(String.valueOf(sysUser.getId()));
        Map<String, Object> map = activitiService.taskApprove(new FlowApproveParam(sopApplyParam.getFlowDeploymentId(), sopApplyParam.getApprove(), approveUserIds, sopApplyParam.getCrmNo(), sopApplyParam.getFormProperties()));
        TFCustomFlow tfCustomFlow = (TFCustomFlow) map.get(SystemConstant.FLOW_CUSTOM);
        TFCustomFlowEntity tfCustomFlowEntity = (TFCustomFlowEntity) map.get(SystemConstant.FLOW_ENTITY);

        TBSopInfo tbSopInfo = new TBSopInfo(sopApplyParam.getCrmNo(), tfCustomFlowEntity.getCode(), sopApplyParam.getServiceId(), sopApplyParam.getCustomId(), sopApplyParam.getProductId(), tfCustomFlow.getType(), sysUser.getId());
        TBSopInfoDetail tbSopInfoDetail = new TBSopInfoDetail(tbSopInfo.getId(), regionUserId, engineerUserId, assistantEngineerUserIds);
        tbSopInfoDetailService.save(tbSopInfoDetail);

        tfCustomFlowEntity.setObjId(tbSopInfo.getId());
        tfCustomFlowEntityService.updateById(tfCustomFlowEntity);
        return tbSopInfoService.save(tbSopInfo);
    }

    /**
     * sop填报信息
     *
     * @param flowApproveParam
     * @return
     */
    @Override
    @Transactional
    public Boolean sopApprove(FlowApproveParam flowApproveParam) throws InterruptedException {
        if (Objects.isNull(flowApproveParam.getTaskId())) {
            throw ExceptionResultEnum.PARAMS_ERROR.exception("流程节点id不能为空");
        }
        if (flowApproveParam.getApprove() != FlowApprovePassEnum.PASS && flowApproveParam.getApprove() != FlowApprovePassEnum.REJECT && flowApproveParam.getApprove() != FlowApprovePassEnum.DRAFT) {
            throw ExceptionResultEnum.PARAMS_ERROR.exception("sop填报只能为通过或驳回或草稿");
        }
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Task task = taskService.createTaskQuery().taskId(String.valueOf(flowApproveParam.getTaskId())).singleResult();
        Optional.ofNullable(task).orElseThrow(() -> ExceptionResultEnum.FLOW_TASK_NO_DATA.exception());

        List<Long> taskApproveUserList = activitiService.getTaskApprove(task);
        if (flowApproveParam.getApprove() != FlowApprovePassEnum.DRAFT && !CollectionUtils.isEmpty(taskApproveUserList) && !taskApproveUserList.contains(sysUser.getId())) {
            throw ExceptionResultEnum.ERROR.exception("不能审批他人的sop数据");
        }

        TFCustomFlowEntity tfCustomFlowEntity = tfCustomFlowEntityService.getOne(new QueryWrapper<TFCustomFlowEntity>().lambda().eq(TFCustomFlowEntity::getFlowId, Long.parseLong(task.getProcessInstanceId())));
        Optional.ofNullable(tfCustomFlowEntity).orElseThrow(() -> ExceptionResultEnum.FLOW_ENTITY_NO_DATA.exception());

        TBSopInfo tbSopInfo = this.getOne(new QueryWrapper<TBSopInfo>().lambda().eq(TBSopInfo::getSopNo, tfCustomFlowEntity.getCode()));
        Optional.ofNullable(tbSopInfo).orElseThrow(() -> ExceptionResultEnum.SOP_INFO_NO_DATA.exception());

        FlowTaskResult flowTaskResult = JSONObject.parseObject(flowApproveParam.getFormProperties(), FlowTaskResult.class);
        Long engineerUserId = null;
        if (flowTaskResult.getSetup().intValue() == 1) {
            Long regionUserId = null;
            String assistantEngineerUserIds = null;
            this.getApproveUserIds(flowTaskResult, regionUserId, engineerUserId, assistantEngineerUserIds);

            UpdateWrapper<TBSopInfoDetail> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().eq(TBSopInfoDetail::getSopInfoId, tbSopInfo.getId())
                    .set(TBSopInfoDetail::getRegionUserId, regionUserId)
                    .set(TBSopInfoDetail::getEngineerUserId, engineerUserId)
                    .set(TBSopInfoDetail::getAssistantEngineerUserIds, assistantEngineerUserIds);
            tbSopInfoDetailService.update(updateWrapper);
        } else {
            engineerUserId = taskApproveUserList.get(0);
        }
        List<String> approveUserIds = Objects.nonNull(engineerUserId) ? Arrays.asList(String.valueOf(engineerUserId)) : Arrays.asList(String.valueOf(sysUser.getId()));
        flowApproveParam.setApproveUserIds(approveUserIds);
        activitiService.taskApprove(flowApproveParam);
        tbSopInfo.updateInfo(sysUser.getId());
        return this.updateById(tbSopInfo);
    }

    /**
     * sop管理列表
     *
     * @param sopInfoListParam
     * @return
     */
    @Override
    public IPage<Map> list(SopInfoListParam sopInfoListParam) {
        return this.baseMapper.list(new Page<>(sopInfoListParam.getPageNumber(), sopInfoListParam.getPageSize()), sopInfoListParam.getServiceId(), sopInfoListParam.getFormWidgetMetadataList());
    }

    /**
     * 获取审批人
     *
     * @param flowTaskResult
     * @param regionUserId
     * @param engineerUserId
     * @param assistantEngineerUserIds
     */
    public void getApproveUserIds(FlowTaskResult flowTaskResult, Long regionUserId, Long engineerUserId, String assistantEngineerUserIds) {
        List<FlowFormWidgetResult> formProperty = flowTaskResult.getFormProperty();
        for (FlowFormWidgetResult f : formProperty) {
            if (f.getFormId().contains(SystemConstant.REGION_USER_ID) && Objects.nonNull(f.getValue())) {
                regionUserId = Long.parseLong(f.getValue());
            }
            if (f.getFormId().contains(SystemConstant.ENGINEER_USER_ID) && Objects.nonNull(f.getValue())) {
                engineerUserId = Long.parseLong(f.getValue());
            }
            if (f.getFormId().contains(SystemConstant.ASSISTANT_ENGINEER_USER_ID) && Objects.nonNull(f.getValue())) {
                String string = f.getValue().replaceAll("\\[", "").replaceAll("\\]", "");
                assistantEngineerUserIds = StringUtils.join(Arrays.asList(string), SystemConstant.LIST_JOIN_SPLIT);
            }
            if (Objects.nonNull(regionUserId) && Objects.nonNull(engineerUserId) && Objects.nonNull(assistantEngineerUserIds)) {
                break;
            }
        }
        if (Objects.isNull(regionUserId)) {
            throw ExceptionResultEnum.ERROR.exception("区域协调人不能为空");
        }
        if (Objects.isNull(engineerUserId)) {
            throw ExceptionResultEnum.ERROR.exception("实施工程师不能为空");
        }
    }
}
